@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2002 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,15 +15,23 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         endpoint.h  -  description
  *                         --------------------------
  *   begin                : Sat Dec 23 2000
- *   copyright            : (C) 2000-2002 by Damien Sandras
- *   description          : This file contains the endpoint methods.
- *   email                : dsandras@seconix.com
+ *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   description          : This file contains the Endpoint class.
  *
  */
 
@@ -31,15 +39,13 @@
 #ifndef _ENDPOINT_H_
 #define _ENDPOINT_H_
 
+#include "../config.h"
+
 #include "common.h"
 #include "gdkvideoio.h"
+#include "videograbber.h"
 #include "lid.h"
 
-#include <ptlib.h>
-#include <h323.h>
-#include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
-
 #ifdef HAS_IXJ
 #include <ixjlid.h>
 #endif
@@ -66,6 +72,15 @@ class GMH323EndPoint : public H323EndPoint
   ~GMH323EndPoint ();
 
 
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Makes a call to the given address, and fills in the
+   *                 call taken. It returns a locked pointer to the connection
+   *                 in case of success.
+   * PRE          :  The called address, its call token.
+   */
+  H323Connection *MakeCallLocked (const PString &, PString &);
+
+  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Updates some of the internal values of the endpoint such 
    *                 as the local username, the capabilities, the tunneling,
@@ -75,6 +90,48 @@ class GMH323EndPoint : public H323EndPoint
    */
   void UpdateConfig ();
 
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Creates a video grabber.
+   * PRE          :  If TRUE, then the grabber will start
+   *                 grabbing after its creation. If TRUE,
+   *                 then the opening is done sync.
+   */  
+  void CreateVideoGrabber (BOOL = true, BOOL = false);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Removes the current video grabber, if any.
+   * PRE          :  If TRUE, then wait until all video grabbers
+   *                 are removed before returning.
+   */  
+  void RemoveVideoGrabber (BOOL = false);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns the current videograbber, if any.
+   *                 The pointer is locked, which means that the device
+   *                 can't be deleted until the pointer is unlocked with
+   *                 Unlock. This is a protection to always manipulate
+   *                 existing objects. Notice that all methods in the
+   *                 GMH323EndPoint class related to the GMVideoGrabber
+   *                 use internal mutexes so that the pointer cannot be
+   *                 returned during a RemoveVideoGrabber/CreateVideoGrabber,
+   *                 among others. You should use those functions and not
+   *                 manually delete the GMVideoGrabber.
+   * PRE          :  /
+   */
+  GMVideoGrabber *GetVideoGrabber ();
+
+  
+  /* DESCRIPTION  :  This callback is called to create an instance of
+   *                 H323Gatekeeper for gatekeeper registration.
+   * BEHAVIOR     :  Return an instance of H323GatekeeperWithNAT
+   *                 that implements the Citron NAT Technology.
+   * PRE          :  /
+   */
+  virtual H323Gatekeeper * CreateGatekeeper (H323Transport *);
+     
   
   /* DESCRIPTION  :  This callback is called if we create a connection
    *                 or if somebody calls and we accept the call.
@@ -101,7 +158,8 @@ class GMH323EndPoint : public H323EndPoint
    * BEHAVIOR     :  Outputs a message in the history and statusbar.
    * PRE          :  /
    */
-  virtual BOOL OnConnectionForwarded (H323Connection &, const PString &,
+  virtual BOOL OnConnectionForwarded (H323Connection &,
+				      const PString &,
 				      const H323SignalPDU &);
 
   
@@ -116,7 +174,23 @@ class GMH323EndPoint : public H323EndPoint
    */
   virtual void OnConnectionEstablished (H323Connection &,
 				        const PString &);
+
   
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns the remote party name (UTF-8), the
+   *                 remote application name (UTF-8), and the best
+   *                 guess for the URL to use when calling back the user
+   *                 (the IP, or the alias or e164 number if the local
+   *                 user is registered to a gatekeeper. Not always accurate,
+   *                 for example if you are called by an user with an alias,
+   *                 but not registered to the same GK as you.)
+   * PRE          :  /
+   */
+  void GetRemoteConnectionInfo (H323Connection &,
+				gchar * &,
+				gchar * &,
+				gchar * &);
+
   
   /* DESCRIPTION  :  This callback is called when the connection to a remote
    *                 endpoint is cleared.
@@ -223,6 +297,14 @@ class GMH323EndPoint : public H323EndPoint
   int GetCallingState (void);
 
 
+  /* Overrides from H323Endpoint */
+  H323Connection *SetupTransfer (const PString &,
+				 const PString &,
+				 const PString &,
+				 PString &,
+				 void * = NULL);
+  
+
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Return the current IP of the endpoint, 
    *                 even if the endpoint is listening on many interfaces
@@ -265,22 +347,6 @@ class GMH323EndPoint : public H323EndPoint
   H323Connection *GetCurrentConnection (void);
 
 
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current audio channel in use
-   *                 NULL if there is no one.
-   * PRE          :  /
-   */
-  H323Channel *GetCurrentAudioChannel (void);
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current video channel in use
-   *                 NULL if there is no one.
-   * PRE          :  /
-   */
-  H323Channel *GetCurrentVideoChannel (void);
-
-  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Return the number of video channels in use.
    *                 0 if there is no one, or if we are not in a call.
@@ -289,22 +355,6 @@ class GMH323EndPoint : public H323EndPoint
   int GetVideoChannelsNumber (void);
 
 
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current video codec in use
-   *                 NULL if there is no one.
-   * PRE          :  /
-   */
-  H323VideoCodec *GetCurrentVideoCodec (void);
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current audio codec in use
-   *                 NULL if there is no one.
-   * PRE          :  /
-   */
-  H323AudioCodec *GetCurrentAudioCodec (void);
-
-  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the current connection to the parameter.
    * PRE          :  A valid pointer to the current connection.
@@ -342,13 +392,6 @@ class GMH323EndPoint : public H323EndPoint
   void GatekeeperRegister (void);
 
 
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current video grabber device thread.
-   * PRE          :  /
-   */
-  PThread *GetVideoGrabberThread (void);
-
-
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Return the current ILS/LDAP client thread.
    * PRE          :  /
@@ -366,16 +409,58 @@ class GMH323EndPoint : public H323EndPoint
 
 #ifdef HAS_IXJ
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current Lid Thread.
+   * BEHAVIOR     :  Returns the current Lid Thread pointer, locked so that
+   *                 the object content is protected against deletion. See
+   *                 GetVideoGrabber ().
    * PRE          :  /
    */
-  GMLid *GetLidThread ();
+  GMLid *GetLid ();
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Removes the current Lid.
+   * PRE          :  /
+   */
+  void RemoveLid ();
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Create a new Lid.
+   * PRE          :  /
+   */
+  void CreateLid ();
+
 #endif
 
 
  protected:
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Set the current transfer call token.
+   * PRE          :  A valid PString for a call token.
+   */
+  void SetTransferCallToken (PString);
+
   
-  PString current_call_token;  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Return the current transfer call token (empty if
+   *                 no call is in progress).
+   * PRE          :  /
+   */
+  PString GetTransferCallToken (void);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns when the primary call of a transfer is terminated.
+   * PRE          :  /
+   */
+  void TransferCallWait ();
+
+
+  PString called_address;
+  PString current_call_token;
+  PString transfer_call_token;
+
   H323Connection *current_connection;  
   H323ListenerTCP *listener;  
 
@@ -383,7 +468,6 @@ class GMH323EndPoint : public H323EndPoint
   int docklet_timeout; 
   int sound_timeout; 
   int display_config; 
-  int snapshot_number;
 
   GDKVideoOutputDevice *transmitted_video_device; 
   GDKVideoOutputDevice *received_video_device; 
@@ -396,18 +480,29 @@ class GMH323EndPoint : public H323EndPoint
   GmTextChat *chat;
   GConfClient *client;
 
+  GMVideoGrabber *video_grabber;
   PThread *ils_client;
-  PThread *video_grabber;
   PThread *audio_tester;
 
-  PMutex var_access_mutex;
-  PMutex quit_mutex;
+  /* Mutexes for the different variables to make
+     sure there is not 2 threads accessing them
+     at the same time */
+  PMutex vg_access_mutex;
+  PMutex ils_access_mutex;
+  PMutex cc_access_mutex;
+  PMutex ch_access_mutex;
+  PMutex cs_access_mutex;
+  PMutex ct_access_mutex;
+  PMutex tct_access_mutex;
+  PMutex lid_access_mutex;
+  
+  PIntCondMutex *vg;
 
   int opened_audio_channels;
   int opened_video_channels;
 
 #ifdef HAS_IXJ
-  GMLid *lid_thread;
+  GMLid *lid;
 #endif
 };
 
